try:
    import matplotlib.widgets
except ImportError:
    pass

"""@package reflex_plot_widgets
This package allows easy interaction between plots using predefined widgets
"""


class CallableWidget :
    """This class is used as a base class for reflex widgets
    It will allow to add a hook to the callback function that is executed
    after the main callback functionality."""
    def __init__(self) :
        self.postcallback = None

    def setPostCallback(self, postcallback):
        self.postcallback = postcallback

    def postCallback(self, new_params):
        if(self.postcallback is not None) :
            self.postcallback(new_params)
            
class InteractiveCheckButtons(CallableWidget) :
    """This class will create a list of check buttons in a matplotlib canvas
    allowing the creator to specify a callback function to be called when
    any of the buttons is clicked or uncliked."""
    def __init__(self, axes, callback, labels, actives, title = None) :
        CallableWidget.__init__(self)
        self.cbuttons = matplotlib.widgets.CheckButtons(axes, labels, actives)
        if title is not None :
            axes.set_title(title, fontsize=12, fontweight='semibold')
        self.cbuttons.on_clicked(self.__callback)
        self.usercallback = callback        
        
    def __callback(self, label):
        new_params = self.usercallback(label)
        self.postCallback(new_params)



class InteractiveSlider(CallableWidget) :
    """This class will create a slider in a matplotlib canvas
    allowing the creator to specify a callback function to be called when
    a slider position is selected."""
    def __init__(self, axes, callback, min,max,value, active, 
                 textbkgcolors = None, title = None) :
        CallableWidget.__init__(self)

        if title is not None :
            axes.set_title(title, fontsize=12, fontweight='semibold')

        textslider=""
        self.slider = matplotlib.widgets.Slider(axes, textslider, min, max, value,dragging=True,valfmt='%g')

        self.slider.on_changed(self.__callback)
        self.usercallback = callback        
        
    def __callback(self, label):
        new_params = self.usercallback(label)
        self.postCallback(new_params)

class InteractiveRadioButtons(CallableWidget) :
    """This class will create a list of radio buttons in a matplotlib canvas
    allowing the creator to specify a callback function to be called when
    a new radio button is selected."""
    def __init__(self, axes, callback, labels, active, 
                 textbkgcolors = None, title = None) :
        CallableWidget.__init__(self)
        if textbkgcolors is not None :
            if len(labels) != len(textbkgcolors) :
                raise Exception('Labels and and background colors sizes do not match')
            textlabels = list()
            for label, bkg in zip(labels, textbkgcolors) :
                text = matplotlib.text.Text(text=label, color = 'blue')
                textlabels.append(text)
        else:
            textlabels = labels
        if title is not None :
            axes.set_title(title, fontsize=12, fontweight='semibold')
        self.rbuttons = matplotlib.widgets.RadioButtons(axes, 
                                                        textlabels, active)
        print "radiobutton"
        self.rbuttons.on_clicked(self.__callback)
        self.usercallback = callback        
        
    def __callback(self, label):
        new_params = self.usercallback(label)
        self.postCallback(new_params)

class InteractiveClickableSubplot(CallableWidget) :
    """This class will create a widget that allows to click with the
       middle button of the mouse in an image and return the coordinates
       in a callback."""

    class AxesWidget(matplotlib.widgets.Widget):
        """NOTE: This class has been copied from matplotlib code 1.2.1 directly.
        matplotlib.widgets.AxesWidget was introduced in version 1.2.
        Once there is no reason to maintain versions of matplotlib older
        than 1.2, matplotlib.widgets.AxesWidget should be used directly.

        Widget that is connected to a single :class:`~matplotlib.axes.Axes`.

        Attributes:

        *ax* : :class:`~matplotlib.axes.Axes`
            The parent axes for the widget
        *canvas* : :class:`~matplotlib.backend_bases.FigureCanvasBase` subclass
            The parent figure canvas for the widget.
        *active* : bool
            If False, the widget does not respond to events.
        """
        def __init__(self, ax):
            self.ax = ax
            self.canvas = ax.figure.canvas
            self.cids = []
            self.active = True

        def connect_event(self, event, callback):
            """Connect callback with an event.

            This should be used in lieu of `figure.canvas.mpl_connect` since this
            function stores call back ids for later clean up.
            """
            cid = self.canvas.mpl_connect(event, callback)
            self.cids.append(cid)

        def disconnect_events(self):
            """Disconnect all events created by this widget."""
            for c in self.cids:
                self.canvas.mpl_disconnect(c)

        def ignore(self, event):
            """Return True if event should be ignored.

            This method (or a version of it) should be called at the beginning
            of any event callback.
            """
            return not self.active

    def __init__(self, axes, callback) :
        CallableWidget.__init__(self)
        #This is what should be used once the private AxesWidget is removed
        #self.axeswidget = matplotlib.widgets.AxesWidget(axes)
        self.axeswidget = self.AxesWidget(axes)
        self.axes = axes
        self.axeswidget.connect_event('button_press_event', self.__callback)
        self.usercallback = callback        

    def __callback(self, point):
        if point.button == 2 :
            if point.inaxes == self.axes : 
                new_params = self.usercallback(point)
                self.postCallback(new_params)




from reflex import RecipeParameter
class Parameters(RecipeParameter):

    # This function specifies which are the parameters that should be presented
    # in the window to be edited.
    # Note that the parameter has to be also in the in_sop port (otherwise it 
    # won't appear in the window) 
    # The descriptions are used to show a tooltip. They should match one to one
    # with the parameter list 
    # Note also that parameters have to be prefixed by the 'recipe name:'

    def setDecodeBpParameters(self,paramList,rec_id,group):

      help1="""Integer representation of the bits to be considered bad when decoding the bad pixel mask pixel values.
                    Most frequent codes relevant for the user:
                    0: good pixel, 
                    8: pick-up noise, 
                    16: cosmic-ray removed, 
                    32: cosmic-ray unremoved, 
                    128: calibration file defect, 
                    256: hot pixel, 
                    512: dark pixel, 
                    4096: A/D converted saturation, 
                    32768: non linear pixel, 
                    1048576: extrapolated flux in NIR, 
                    4194304: Interpolated flux during extraction."""
      
      
      paramList.append(RecipeParameter(rec_id,displayName="decode-bp",group=group,description=help1))


    def setPreOverscanParameters(self,paramList,rec_id,group):

      help2="""pre-overscan correction.0: no correction1: mean overscan correction2: mean prescan correction3: (mean pre+mean overscan)/2 correction."""
      paramList.append(RecipeParameter(rec_id,displayName="pre-overscan-corr",group=group,description=help2))


    def setGeneralCalibParameters(self,paramList,rec_id):
      group="general"

      self.setPreOverscanParameters(paramList,rec_id,group)


    def setGeneralObjectParameters(self,paramList,rec_id):

      group="general"
      print "rec_id=",rec_id

      self.setDecodeBpParameters(paramList,rec_id,group)
      self.setPreOverscanParameters(paramList,rec_id,group)

      help5="""if TRUE recompute (wave and slit) maps from the dispersion solution. Use FALSE only if you are sure to have wave and slit maps better than what this recipe may compute.""" 


      help6="""if TRUE compute object position trace via Gaussian fit."""
      help7="""Method used for extraction (LOCALIZATION, NOD)."""
      help8="""if TRUE a 2D sky frame, a 2D rectified, a 2D merged sky are generated."""
      help9="""Lower Slit Limit (localize and extract."""
      help10="""Upper Slit Limit (localize and extract."""


      if "stare" in rec_id:
        print "stare"
        help5=help5+"If sky-subtract is set to TRUE this must be set to TRUE."
        paramList.append(RecipeParameter(rec_id,displayName="compute-map",group=group,description=help5))
        paramList.append(RecipeParameter(rec_id,displayName="trace-obj",group=group,description=help6))

      if "nod" in rec_id :
        print "extract method"
        paramList.append(RecipeParameter(rec_id,displayName="extract-method",group=group,description=help7))

      if "slit_offset" in rec_id :
        paramList.append(RecipeParameter(rec_id,displayName="gen-sky",group=group,description=help8))

      if "slit_nod" in rec_id :
        paramList.append(RecipeParameter(rec_id,displayName="max-slit",group=group,description=help9))
        paramList.append(RecipeParameter(rec_id,displayName="min-slit",group=group,description=help10))

        help14="""Combination method for nodded frames (MEDIAN, MEAN)."""
        paramList.append(RecipeParameter(rec_id,displayName="combinenod-method",group=group,description=help14))

        help15="""TRUE if the resampled spectrum at each wavelength is median subtracted to remove sky lines."""
        paramList.append(RecipeParameter(rec_id,displayName="correct-sky-by-median",group=group,description=help15))

      if "slit" in rec_id :

         help12="""Half size of mask used to define object cross orderprofile. [7]. [FALSE]."""
         paramList.append(RecipeParameter(rec_id,displayName="stdextract-interp-hsize",group=group,description=help12))

      if rec_id[0:10] == "xsh_respon":
         help13="""TRUE if during response computation we apply telluric correction. [FALSE]."""
         paramList.append(RecipeParameter(rec_id,displayName="correct-tellurics",group=group,description=help13))

      if rec_id[0:10] == "xsh_scired":
         help16="""TRUE if recipe cuts the UVB spectrum at 556 nm (dichroich). [TRUE]."""
         paramList.append(RecipeParameter(rec_id,displayName="cut-uvb-spectrum",group=group,description=help16))

      if "slit" in rec_id :
         help17="""TRUE if additional files should be generated in Science Data Product (SDP) format. [FALSE]."""
         paramList.append(RecipeParameter(rec_id,displayName="generate-SDP-format",group=group,description=help17))

      return paramList







    def setDetectArclinesParameters(self,paramList,rec_id):
      group="detectarclines"
      help1="""Half window size (HWS) in pixels for the line 2D fitting window (total window size = 2*HWS+1)."""
      paramList.append(RecipeParameter(rec_id,displayName="detectarclines-fit-win-hsize",group=group,description=help1))

      help2="""Half window size (HWS) in pixels for the line search box around the expected position (total window size = 2*HWS+1) [bin units]."""

      paramList.append(RecipeParameter(rec_id,displayName="detectarclines-search-win-hsize",group=group,description=help2))


      if rec_id[4:11] != "predict" :
         help7="""Degree in Y in the polynomial order tracing X=f(Y)."""
         paramList.append(RecipeParameter(rec_id,displayName="detectarclines-ordertab-deg-y",group=group,description=help7))


      help8="""Minimum signal-to-noise ratio to filter lines"""
      paramList.append(RecipeParameter(rec_id,displayName="detectarclines-min-sn",group=group,description=help8))


    def setDetectContinuumParameters(self,paramList,rec_id):
      group="detectcontinuum"

      help1="""Half window size for the running median box during the search for the maximum in the cross-dispersion profile."""
      paramList.append(RecipeParameter(rec_id,displayName="detectcontinuum-search-win-hsize",group=group,description=help1))

      help2="""Half window size for the fit of the cross-dispersion profile."""
      paramList.append(RecipeParameter(rec_id,displayName="detectcontinuum-running-win-hsize",group=group,description=help2))

      help3="""Threshold factor applied to check that the flux at the fitted peak is higher than error."""
      paramList.append(RecipeParameter(rec_id,displayName="detectcontinuum-fit-win-hsize",group=group,description=help3))

      help4="""Degree in Y in the polynomial order tracing X=f(Y)."""
      paramList.append(RecipeParameter(rec_id,displayName="detectcontinuum-ordertab-step-y",group=group,description=help4))

      help5="""Maximum allowed residual (before kappa-sigma clip)."""
      paramList.append(RecipeParameter(rec_id,displayName="detectcontinuum-ordertab-deg-y",group=group,description=help5))


    def setDetectOrderParameters(self,paramList,rec_id):
      group="detectorder"

      help1="""Half window size in pixels for the search for order edges."""
      paramList.append(RecipeParameter(rec_id,displayName="detectorder-edges-search-win-hsize",group=group,description=help1))

      help2="""Threshold in relative flux (compared to the central flux) below which the order edges are defined."""
      paramList.append(RecipeParameter(rec_id,displayName="detectorder-edges-flux-thresh",group=group,description=help2))

      help3="""Minimum signal-to-noise ratio at the centroid of the orders (35 for SLIT-UVB,VIS, 20 for IFU-UVB,VIS, 25 for SLIT-NIR, 4 for IFU-NIR."""
      paramList.append(RecipeParameter(rec_id,displayName="detectorder-min-sn",group=group,description=help3))

      help11="""minimum signal noise ratio in order."""
      paramList.append(RecipeParameter(rec_id,displayName="detectorder-d2-min-sn",group=group,description=help11))


    def setBackgroundParameters(self,paramList,rec_id):
      group="background"


      help5="""Poly mode fit deg along X."""
      help6="""Poly mode fit deg along Y."""
      help7="""Poly mode kappa value of kappa-sigma-clip outliers removal."""
      help8="""X margin to order edge to define background sampling points."""

      paramList.append(RecipeParameter(rec_id,displayName="background-poly-deg-x",group=group,description=help5))
      paramList.append(RecipeParameter(rec_id,displayName="background-poly-deg-y",group=group,description=help6))
      paramList.append(RecipeParameter(rec_id,displayName="background-poly-kappa",group=group,description=help7))
      paramList.append(RecipeParameter(rec_id,displayName="background-edges-margin",group=group,description=help8))



    def setModelParameters(self,paramList,rec_id):
      group="model"

      help1="""Number/10 of annealing iterations if in physical model mode."""
      paramList.append(RecipeParameter(rec_id,displayName="model-maxit",group=group,description=help1))

      help2="""Multiplier applied to the automatic parameter ranges (i.e. when scenario!=0). For routine operations should be 1.0. (physical model mode)."""
      paramList.append(RecipeParameter(rec_id,displayName="model-anneal-factor",group=group,description=help2))

      help3="""selects preset flag and range combinations appropriate to common scenarios: 0 - No scenario, input cfg flags and limits used.1 - scenario appropriate for the startup recipe (large ranges for parameters affecting single ph exposures, dist coeff fixed).  2 - Like 1, but includes parameters affecting all ph positions.  3 - Scenario for use in fine tuning cfg to match routine single pinhole exposures. All parameters affecting 1ph exposures except dist coeffs are included and parameter ranges are small. (For use by predict in 1ph case).  4 - Like 3 but includes parameters  affecting all ph positions (Standard for use by predict in 9ph case and 2dmap)."""


    def setStackParameters(self,paramList,rec_id):
      group="stack"
      help1="""Method used to build master frame. <median | mean>"""
      paramList.append(RecipeParameter(rec_id,displayName="stack-method",group=group,description=help1))

      help2="""Kappa used to clip low level values, when method is set to 'mean'."""
      paramList.append(RecipeParameter(rec_id,displayName="klow",group=group,description=help2))

      help3="""Kappa used to clip high level values, when method is set to 'mean'."""
      paramList.append(RecipeParameter(rec_id,displayName="khigh",group=group,description=help3))

      help4="""Number of kappa sigma iterations, when method is set to 'mean'."""


    def setCrhSingleParameters(self,paramList,rec_id):
      group="removecrhsingle"
      help1="""Max fraction of bad pixels allowed."""
      help2="""Poisson fluctuation threshold to flag CRHs (see van Dokkum, PASP,113,2001,p1420-27)."""
      help3="""Minimum contrast between the Laplacian image and the fine structure image that a point must have to be flagged as CRH. (see van Dokkum, PASP, 113, 2001, p1420-27)."""
      help4="""Max number of iterations."""
      paramList.append(RecipeParameter(rec_id,displayName="removecrhsingle-frac-max",group=group,description=help1))
      paramList.append(RecipeParameter(rec_id,displayName="removecrhsingle-sigmalim",group=group,description=help2))
      paramList.append(RecipeParameter(rec_id,displayName="removecrhsingle-flim",group=group,description=help3))
      paramList.append(RecipeParameter(rec_id,displayName="removecrhsingle-niter",group=group,description=help4))


    def setLocalizeParameters(self,paramList,rec_id):
      group="localize"
      help1="""Localization method (MANUAL, MAXIMUM, GAUSSIAN) used to detect the object centroid and height on the slit."""
      help2="""Number of chunks in the full spectrum to localize the object."""
      help3="""Threshold relative to the peak intensity below which the edges of the object are detected for MAXIMUM localization."""
      help4="""Degree in lambda in the localization polynomial expression slit=f(lambda), used only for MAXIMUM and GAUSSIAN."""
      help5="""Object position on the slit for MANUAL localization [arcsec]."""
      help6="""Object half height on the slit for MANUAL localization [arcsec]."""
      help7="""Kappa value for sigma clipping in the localization polynomial fit."""
      help8="""Number of iterations for sigma clipping in the localization polynomial fit."""
      help9="""TRUE if we want to mask sky lines using SKY_LINE_LIST file."""
      help10="""Step (arcsec) between A and B images in nodding mode."""

      paramList.append(RecipeParameter(rec_id,displayName="localize-method",group=group,description=help1))
      paramList.append(RecipeParameter(rec_id,displayName="localize-chunk-nb",group=group,description=help2))
      paramList.append(RecipeParameter(rec_id,displayName="localize-thresh",group=group,description=help3))
      paramList.append(RecipeParameter(rec_id,displayName="localize-deg-lambda",group=group,description=help4))
      paramList.append(RecipeParameter(rec_id,displayName="localize-slit-position",group=group,description=help5))
      paramList.append(RecipeParameter(rec_id,displayName="localize-slit-hheight",group=group,description=help6))
      paramList.append(RecipeParameter(rec_id,displayName="localize-kappa",group=group,description=help7))
      paramList.append(RecipeParameter(rec_id,displayName="localize-niter",group=group,description=help8))
      paramList.append(RecipeParameter(rec_id,displayName="localize-use-skymask",group=group,description=help9))
      if rec_id == "xsh_scired_slit_nod" or rec_id == "xsh_respon_slit_nod":
         paramList.append(RecipeParameter(rec_id,displayName="localize-nod-throw",group=group,description=help10))

    def setRectifyParameters(self,paramList,rec_id):
      group="rectify"
      help1="""Name of the Interpolation Kernel Used. Possible values are tanh, tanh, sinc, sinc2, lanczos, hamming, hann."""
      help2="""Rectify Interpolation radius [bin units]."""
      help3="""Wavelength step in the output spectrum [nm]."""
      help4="""Spatial step along the slit in the output spectrum [arcsec]."""

      paramList.append(RecipeParameter(rec_id,displayName="rectify-kernel",group=group,description=help1))
      paramList.append(RecipeParameter(rec_id,displayName="rectify-radius",group=group,description=help2))
      paramList.append(RecipeParameter(rec_id,displayName="rectify-bin-lambda",group=group,description=help3))
      paramList.append(RecipeParameter(rec_id,displayName="rectify-bin-slit",group=group,description=help4))


    def setOptExtractParameters(self,paramList,rec_id):
      group="optextract"
      help1="""TRUE if we do the optimal extraction."""
      help2="""Oversample factor for the science image."""
      help3="""Extraction box [pixel]."""
      help4="""Chunk size [bin]."""
      help5="""Lambda step [nm]."""
      help6="""Kappa for cosmics ray hits rejection."""
      help7="""Maximum bad pixels fraction for cosmics ray hits rejection."""
      help8="""Maximum number of iterations for cosmics ray hits rejection."""
      help9="""Number of iterations."""
      help10="""Extraction method GAUSSIAN | GENERAL."""


      paramList.append(RecipeParameter(rec_id,displayName="do-optextract",group=group,description=help1))
      paramList.append(RecipeParameter(rec_id,displayName="optextract-oversample",group=group,description=help2))
      paramList.append(RecipeParameter(rec_id,displayName="optextract-box-half-size",group=group,description=help3))
      paramList.append(RecipeParameter(rec_id,displayName="optextract-chunk-size",group=group,description=help4))
      paramList.append(RecipeParameter(rec_id,displayName="optextract-step-lambda",group=group,description=help5))
      paramList.append(RecipeParameter(rec_id,displayName="optextract-clip-kappa",group=group,description=help6))
      paramList.append(RecipeParameter(rec_id,displayName="optextract-clip-frac",group=group,description=help7))
      paramList.append(RecipeParameter(rec_id,displayName="optextract-clip-niter",group=group,description=help8))
      paramList.append(RecipeParameter(rec_id,displayName="optextract-niter",group=group,description=help9))
      paramList.append(RecipeParameter(rec_id,displayName="optextract-method",group=group,description=help10))


    def setSkyParameters(self,paramList,rec_id):
      group="sky"
      help1="""TRUE to use subtract sky single."""
      help2="""Sky subtract Method (BSPLINE, MEDIAN)."""
      help3="""Nb of break points for Bezier curve fitting (without localization)."""
      help4="""Nb of break points for Bezier curve fitting (with localization).""" 
      help5="""Bezier spline order."""
      help6="""Nb of iterations. [20]"""
      help7="""Size of edges mask in arcsec."""
      help8="""Half size of the running median. If sky-method=MEDIAN."""
      help9="""Size of edges mask in arcsec."""
      help10="""Central position of the sky window #1 [arcsec]."""
      help11="""Half size of sky window #1 [arcsec]."""
      help12="""Central position of the sky window #2 [arcsec]."""
      help13="""Half size of the sky window #2 [arcsec]."""
      help14="""BSPLINE sampling. UNIFORM-uses the user defined nbkpts value, corrected for binning, for all orders. FINE: multiplies the user defined nbkpts value, corrected for binning, by a hard coded coefficient optimized on each arm-order)."""
        
      paramList.append(RecipeParameter(rec_id,displayName="sky-subtract",group=group,description=help1))
      paramList.append(RecipeParameter(rec_id,displayName="sky-method",group=group,description=help2))

      paramList.append(RecipeParameter(rec_id,displayName="sky-bspline-nbkpts-first",group=group,description=help3))
      paramList.append(RecipeParameter(rec_id,displayName="sky-bspline-nbkpts-second",group=group,description=help4))
      paramList.append(RecipeParameter(rec_id,displayName="sky-bspline-order",group=group,description=help5))
      paramList.append(RecipeParameter(rec_id,displayName="sky-bspline-niter",group=group,description=help6))
      paramList.append(RecipeParameter(rec_id,displayName="sky-bspline-kappa",group=group,description=help7))
      paramList.append(RecipeParameter(rec_id,displayName="sky-median-hsize",group=group,description=help8))
      paramList.append(RecipeParameter(rec_id,displayName="sky-slit-edges-mask",group=group,description=help9))
      paramList.append(RecipeParameter(rec_id,displayName="sky-position1",group=group,description=help10))
      paramList.append(RecipeParameter(rec_id,displayName="sky-hheight1",group=group,description=help11))
      paramList.append(RecipeParameter(rec_id,displayName="sky-position2",group=group,description=help12))
      paramList.append(RecipeParameter(rec_id,displayName="sky-hheight2",group=group,description=help13))
      paramList.append(RecipeParameter(rec_id,displayName="bspline-sampling",group=group,description=help14))



# import the needed modules
try:
#  import sys
  import numpy
  try:
    from astropy.io import fits as pyfits
  except:
    import pyfits

  import xsh_parameters_common
#  import wx
#  import matplotlib
  import reflex
  #from xsh_plot_common import *
  from reflex import parseSofJson,RecipeParameter
  from reflex_interactive_app import PipelineInteractiveApp
  from pipeline_product import PipelineProduct
  from pipeline_display import SpectrumDisplay, ImageDisplay, ScatterDisplay
#  matplotlib.use('WXAgg')
  import_sucess = 'true'
  import warnings
  warnings.simplefilter('ignore',UserWarning)

#NOTE for developers: 
# -If you want to modify the current script to cope
#  with different parameters, this is the function to modify:
#  setInteractiveParameters()
# -If you want to modify the current script to read different data from
#  the input FITS, these are the functions (class DataPlotterManager) to modify:
#  readFitsData()    to indicate what are the FITS input to plot
#  plotSpectrum()    to control 1D image product layout
#  plotImage()       to control 2D image product layout
#  plotTable()       to control 1D table product layout
#  oplotTable()      to control 1D table product overplots
#  setRecId()        to set rec id (used in parameter prefix)
#  getArmId()        to get arm id FITS header info
#
# -If you want to modify the current script to modify the plots (using the same
#  data),  this is the function to modify:
#  plotProductsGraphics()          (from class DataPlotterManager)
# -If you want to modify the text that appears in the "Help" button,
#  this is the function to modify:
#  setWindowHelp()
# -If you want to modify the title of the window, modify this function:
#  setWindowTitle()


  #This class deals with the specific details of data reading and final plotting.
  class DataPlotterManager:

    def paragraph(self,text, width=None):
       """ wrap text string into paragraph
           text:  text to format, removes leading space and newlines
           width: if not None, wraps text, not recommended for tooltips as
                  they are wrapped by wxWidgets by default
       """
       import textwrap
       #print 'width=',width,'Text=',text
       if width is None:
           return textwrap.dedent(text).replace('\n', ' ').strip()
       else:
           return textwrap.fill(textwrap.dedent(text), width=width) 


    #1D spectrum plots
    def plotSpectrum(self,data,errs,subplot,title,tooltip):
        data.readSpectrum(0)
        errs.readSpectrum(1)
        spectrum = SpectrumDisplay()
        spectrum.setLabels('Wavelength [nm]','Total Flux ['+data.bunit+']')
        spectrum.display(subplot,title,self.paragraph(tooltip),data.wave,data.flux,errs.flux, autolimits = True) 

    #2D image display
    def plotImage(self,obj,bpm,subplot,title,tooltip):

          obj.readImage(fits_extension=0)
          obj.read2DLinearWCS(fits_extension=0)
          img_obj = ImageDisplay()

          bpm.readImage(fits_extension=2)
          img_bpm = ImageDisplay()

	  img_obj.setLabels('Wavelength [nm]','Position Along Slit [arcsec]')
          y_size, x_size = obj.image.shape

          img_obj.setXLinearWCSAxis(obj.crval1,obj.cdelt1,obj.crpix1)
          img_obj.setYLinearWCSAxis(obj.crval2,obj.cdelt2,obj.crpix2)

	  #img_obj.setLimits((1,x_size),(0,y_size))
          img_obj.display(subplot, title, self.paragraph(tooltip),obj.image,bpm.image)

    def oplotImageScatter(self,img,tab,ext,binx,biny,Xcol,Ycol,subplot, marker = 'o', size = 10, color = 'blue' ):
          tab.readTableXYColumns(fits_extension=ext,xcolname=Xcol, ycolname=Ycol)


          py_to_FITS_off_x=0
          py_to_FITS_off_y=0
          label_off_x=0
          label_off_y=0
          #print 'offset=',label_off_x,label_off_y,py_to_FITS_off_x,py_to_FITS_off_y

          img.overplotScatter(tab.x_column/binx+py_to_FITS_off_x, tab.y_column/biny+py_to_FITS_off_y, marker, size, color)

    def prepImageScatter(self,img_disp,obj,bpm,Xlab,Ylab):

          obj.readImage(fits_extension=0)
          obj.read2DLinearWCS(fits_extension=0)

          bpm.readImage(fits_extension=2)
          img_bpm = ImageDisplay()

	  img_disp.setLabels(Xlab,Ylab)
          y_size, x_size = obj.image.shape

          img_disp.setXLinearWCSAxis(obj.crval1,obj.cdelt1,obj.crpix1)
          img_disp.setYLinearWCSAxis(obj.crval2,obj.cdelt2,obj.crpix2)

	  #self.img_obj.setLimits((0,x_size),(0,y_size))


    #2D image display
    def plotImageScatter1Col(self,obj,bpm,tab,ext,binx,biny,Xlab,Ylab,Xcol1,Ycol1,subplot,title,tooltip):

          img_disp = ImageDisplay()

          self.prepImageScatter(img_disp,obj,bpm,Xlab,Ylab)

          self.oplotImageScatter(img_disp,tab,ext,binx,biny,Xcol1,Ycol1,subplot, 'o', 10, 'green' )

          bpm_filt = ( (bpm.image.astype(int) & self.decode_bp) != 0 )
          img_disp.display(subplot, title, self.paragraph(tooltip),obj.image,bpm_filt)
          #as the scatter display add extra space round the image (not
          #to have points at edges) we force the bounds of the scatter
          #plot to be the same as the image size.
          #subplot.set_xbound(0,obj.image.shape[1])
          #subplot.set_ybound(0,obj.image.shape[0])

    #2D image display
    def plotImageScatter2Col(self,obj,bpm,tab,ext,binx,biny,Xlab,Ylab,Xcol1,Ycol1,Xcol2,Ycol2,subplot,title,tooltip):

          img_disp = ImageDisplay()

          self.prepImageScatter(img_disp,obj,bpm,Xlab,Ylab)

          self.oplotImageScatter(img_disp,tab,ext,binx,biny,Xcol1,Ycol1,subplot, 'o', 10, 'blue' )
          self.oplotImageScatter(img_disp,tab,ext,binx,biny,Xcol2,Ycol2,subplot, 'o', 10, 'yellow' )

          bpm_filt = ( (bpm.image.astype(int) & self.decode_bp) != 0 )
          img_disp.display(subplot, title, self.paragraph(tooltip),obj.image,bpm_filt)
          #as the scatter display add extra space round the image (not
          #to have points at edges) we force the bounds of the scatter
          #plot to be the same as the image size.
          #subplot.set_xbound(0,obj.image.shape[1])
          #subplot.set_ybound(0,obj.image.shape[0])

    #2D image display
    def plotImageScatter3Col(self,obj,bpm,tab,ext,binx,biny,Xlab,Ylab,Xcol1,Ycol1,Xcol2,Ycol2,Xcol3,Ycol3,subplot,title,tooltip):

          img_disp = ImageDisplay()

          self.prepImageScatter(img_disp,obj,bpm,Xlab,Ylab)


          self.oplotImageScatter(img_disp,tab,ext,binx,biny,Xcol1,Ycol1,subplot, 'o', 10, 'blue' )
          self.oplotImageScatter(img_disp,tab,ext,binx,biny,Xcol2,Ycol2,subplot, 'o', 10, 'yellow' )
          self.oplotImageScatter(img_disp,tab,ext,binx,biny,Xcol3,Ycol3,subplot, 'o', 10, 'green' )

#          self.img_obj.setXLinearWCSAxis(img_obj.crval1,img_obj.cdelt1,img_obj.crpix1)

          bpm_filt = ( (bpm.image.astype(int) & self.decode_bp) != 0 )
          img_disp.display(subplot, title,self.paragraph(tooltip),obj.image,bpm_filt)
          #as the scatter display add extra space round the image (not
          #to have points at edges) we force the bounds of the scatter
          #plot to be the same as the image size.
          #subplot.set_xbound(0,obj.image.shape[1])
          #subplot.set_ybound(0,obj.image.shape[0])


    #1D spectrum table plots
    def plotTable(self,tab,Xcol,Ycol,Xlab,Ylab,subplot,title,tooltip,
    wave = None):

        tab.readTableXYColumns(fits_extension=1,xcolname=Xcol, ycolname=Ycol)
        spectrum = SpectrumDisplay()
        flux_plotmax = 1.2 * numpy.nanmax(tab.y_column)
        flux_plotmin = 0.0
        spectrum.setLabels(Xlab,Ylab)
        spectrum.flux_lim = flux_plotmin, flux_plotmax
        if wave is not None:
           spectrum.setWaveLimits((wave[0], wave[len(wave)-1]))

        spectrum.display(subplot,title,self.paragraph(tooltip),tab.x_column,tab.y_column) 

    #1D spectrum table over-plots
    def oplotTable(self,tab,Xcol,Ycol,Xlab,Ylab,subplot,color):

        tab.readTableXYColumns(fits_extension=1,xcolname=Xcol, ycolname=Ycol)
        spectrum = SpectrumDisplay()
        spectrum.overplot(subplot,tab.x_column,tab.y_column,color)


    def scatterPlotScatter(self, subplot, x, y):
        subplot.scatter(x, y, s=0.4, color='blue')
        subplot.ticklabel_format(style='sci',scilimits=(0,3))

    def scatterPlotLabels(self, subplot, x_label, y_label):
        subplot.set_xlabel(x_label, fontsize=12)
        subplot.set_ylabel(y_label, fontsize=12)

    def finalSetup(self, subplot, title, tooltip):
        subplot.grid(True)
        subplot.set_title(title, fontsize=12)
        subplot.tooltip = self.paragraph(tooltip)


    def scatterPlot(self, subplot, x, y, x_label, y_label, title, tooltip):

        self.scatterPlotScatter(subplot, x, y)
        self.scatterPlotLabels(subplot, x_label, y_label)
        self.scatterPlotLimits(subplot, x, y)
        self.finalSetup(subplot, title, self.paragraph(tooltip))

    def scatterPlotLimits(self, subplot, x, y):
        subplot.set_xlim(numpy.nanmin(x)-1, numpy.nanmax(x) + 1.)
        subplot.set_ylim(1.05 * numpy.nanmin(y), 1.05 * numpy.nanmax(y))



    #1D table scatter plots
    def plotTableScatter(self,tab,Xcol,Ycol,Xlab,Ylab,subplot,title,tooltip,ext_no=None):
        extno=1
        if ext_no != None :
           extno=ext_no

        tab.readTableXYColumns(fits_extension=extno,xcolname=Xcol,ycolname=Ycol)
        self.scatterPlotScatter(subplot, tab.x_column, tab.y_column)
        self.scatterPlotLabels(subplot, Xlab, Ylab)
        self.scatterPlotLimits(subplot, tab.x_column, tab.y_column)
        self.finalSetup(subplot, title, self.paragraph(tooltip))
        self.scatterPlot(subplot, tab.x_column, tab.y_column, Xlab, Ylab, title, tooltip)


    # This function will read all the columns, images and whatever is needed
    # from the products. The variables , self.plot_x, self.plot_y, etc...
    # are used later in function plotProductsGraphics().
    # Add/delete these variables as you need (only that plotProductsGraphics()
    # has to use the same names).
    # You can also create some additional variables (like statistics) after
    # reading the files.
    # If you use a control variable (self.xxx_found), you can modify 
    # later on the layout of the plotting window based on the presence of 
    # given input files. 
    # sof contains all the set of frames
    def readFitsData(self, fitsFiles):
      #Control variable to check if the interesting files where at the input
      prefix = "CAL_"
      self.resid_tab_found   = False
      self.edges_tab_found   = False
      self.mflat_found = False
      self.frameON_found      = False
      self.grid_back_found      = False
      self.sub_back_found   = False
      self.obj_sky2D_found   = False
      self.bpm_found   = False
      self.arm_uvb_found      = False
      self.arm_vis_found      = False
      self.arm_nir_found      = False
      self.nima = 0
      self.binx = 1
      self.biny = 1
      
      #Read all the products
      frames = dict()
      for frame in fitsFiles:
        if frame == '' :
          continue

        header = pyfits.open(frame.name)
        #Make sure to have only products from the same recipe
        #that used this (common) script 
        if 'ESO PRO REC1 ID' in header[0].header :
           rec_id = header[0].header['ESO PRO REC1 ID']
           if rec_id == self.rec_id :
              category = frame.category
              frames[category] = frame


      prefixes = [k.split('_')[0] for k in frames.keys()]
      if "FMTCHK" in prefixes and self.rec_id == "xsh_predict" :
        prefix = "FMTCHK_"
      elif "WAVE" in prefixes and self.rec_id == "xsh_2dmap":
        prefix = "WAVE_"
      elif "FLAT" in prefixes and self.rec_id == "xsh_mflat":
        prefix = "MFLAT_"
      elif "ORDERDEF" in prefixes and self.rec_id == "xsh_orderpos":
        prefix = "ORDERDEF_"
      elif "FLEX" in prefixes and self.rec_id == "xsh_flexcomp":
        prefix = "FLEX_"
 
      arm=self.seq_arm

      if arm == "UVB":
 	 self.arm_uvb_found = True
      if arm == "VIS":
 	 self.arm_vis_found = True
      if arm == "NIR":
 	 self.arm_nir_found = True

# For any arm search a list of input frames
      #print "arm=",arm , "prefix=",prefix 
      #print frames
      key = prefix+"RESID_TAB_LINES_"+arm
      if key in frames :
        self.resid_tab_found = True
        hdulist = frames[prefix+"RESID_TAB_LINES_"+arm]
        self.resid_tab = PipelineProduct(hdulist)
        #self.resid_tab_2plot = PlotableResidLineTable(hdulist,1)
        #self.nima += 1
        #print "found resid tab"

      key = "ORDERPOS_RESID_TAB_"+arm
      if key in frames :
        self.resid_tab_found = True
        hdulist = frames["ORDERPOS_RESID_TAB_"+arm]
        self.resid_tab = PipelineProduct(hdulist)
        #self.resid_tab_2plot = PlotableResidLineTable(hdulist,1)
        #self.nima += 1
        #print "found orderpos resid tab"


      key = "ORDER_TAB_EDGES_SLIT_"+arm
      if key in frames :
        self.edges_tab_found = True
        hdulist = frames["ORDER_TAB_EDGES_SLIT_"+arm]
        self.edges_tab = PipelineProduct(hdulist)
        #self.nima += 1
        #print "found edges tab"

      key = "ORDER_TAB_EDGES_IFU_"+arm
      if key in frames :
        self.edges_tab_found = True
        hdulist = frames["ORDER_TAB_EDGES_IFU_"+arm]
        self.edges_tab = PipelineProduct(hdulist)
        #self.nima += 1
        #print "found edges tab"
	
      key = prefix+"ON_"+arm
      if key in frames :
        self.frameON_found = True
        hdulist = frames[prefix+"ON_"+arm]
        self.bpm = PipelineProduct(hdulist)
        self.frameON = PipelineProduct(hdulist)
        #self.nima += 1
        #print "found on frame"

      key = "MASTER_FLAT_SLIT_"+arm
      if key in frames :
        self.mflat_found = True
        fitsfile = frames["MASTER_FLAT_SLIT_"+arm]
        self.bpm = PipelineProduct(fitsfile)
        self.mflat = PipelineProduct(fitsfile)
        hdulist = pyfits.open(fitsfile.name)
        #print "found mflat frame",self.mflat_found
        #print "Bin setting",self.binx,self.biny

      key = "MASTER_FLAT_IFU_"+arm
      if key in frames :
        self.mflat_found = True
        fitsfile = frames["MASTER_FLAT_IFU_"+arm]
        self.bpm = PipelineProduct(fitsfile)
        self.mflat = PipelineProduct(fitsfile)
        hdulist = pyfits.open(fitsfile.name)
        #print "found mflat frame",self.mflat_found
        #print "Bin setting",self.binx,self.biny


      key = prefix+"GRID_BACK_SLIT_"+arm
      if key in frames :
        self.grid_back_found = True
        hdulist = frames[prefix+"GRID_BACK_SLIT_"+arm]
        self.grid_back = PipelineProduct(hdulist)
        #print "found grid background"

      key = prefix+"GRID_BACK_IFU_"+arm
      if key in frames :
        self.grid_back_found = True
        hdulist = frames[prefix+"GRID_BACK_IFU_"+arm]
        self.grid_back = PipelineProduct(hdulist)
        #print "found grid background"

      key = prefix+"SUB_BACK_SLIT_"+arm
      if key in frames :
        self.sub_back_found = True
        hdulist = frames[prefix+"SUB_BACK_SLIT_"+arm]
        self.sub_back = PipelineProduct(hdulist)
        #print "found background subtracted frame"

      #print "monitor mflat",self.mflat_found

    #Set rec id (to have proper recipe parameters prefix)
    def setRecId(self, rec_id):
      #Recipe ID variable to properly define params
      self.rec_id = rec_id

    #Get arm setting
    def getArmId(self, sof):
      #Recipe ID variable to properly define params
      self.rec_id = "xsh_predict"
      self.seq_arm = "UVB"
      nf = 0

      frames = dict()
      files = sof.files
      for f in files:
        frame=f.name
        if frame == '' :
          continue
        else :
           nf += 1
           hdulist = pyfits.open(frame)
           rec_id_list = hdulist[0].header['ESO PRO REC1 ID']
           arm_list = hdulist[0].header['ESO SEQ ARM']
      if nf != 0 :
         self.rec_id = rec_id_list[0:]
         self.seq_arm = arm_list[0:]

      if self.seq_arm == "UVB":
         self.arm_uvb_found = True

      if self.seq_arm == "VIS":
         self.arm_uvb_found = True

      if self.seq_arm == "NIR":
         self.arm_uvb_found = True

      if self.rec_id == "xsh_predict":
         self.rec_subtitle = "Spectral Format. "
      if self.rec_id == "xsh_orderpos":
         self.rec_subtitle = "Order Position. "
      if self.rec_id == "xsh_mflat":
         self.rec_subtitle = "Master Flat Creation. "
      if self.rec_id == "xsh_2dmap":
         self.rec_subtitle = "2D Geometry. "
      self.decode_bp = int(hdulist[0].header['ESO PRO REC1 PARAM4 VALUE'])
      
    # This function creates all the subplots. It is responsible for the plotting 
    # layouts. 
    # There can different layouts, depending on the availability of data
    # Note that subplot(I,J,K) means the Kth plot in a IxJ grid 
    # Note also that the last one is actually a box with text, no graphs.
    def addSubplots(self, figure):
      nrows=self.nima
      row=1;
      #print "nrows=",nrows,row
      #print "resid_tab_found",self.resid_tab_found, "frameON_found",self.frameON_found
      if self.resid_tab_found == True and self.frameON_found == True:
        #self.subplot_frameON  = figure.add_subplot(nrows,1,row)
        #self.subplot_resid_tab  = figure.add_subplot(nrows,2,row+1)
        self.subplot_frameON  = figure.add_subplot(2,1,1)
        self.subplot_resid_x_tab  = figure.add_subplot(2,2,3)
        self.subplot_resid_y_tab  = figure.add_subplot(2,2,4)
        row +=1

      elif self.edges_tab_found == True and self.mflat_found == True:
        #self.subplot_mflat  = figure.add_subplot(nrows,1,row)
        self.subplot_mflat  = figure.add_subplot(1,1,1)
        row +=1
        
        #print "monitor mflat",self.mflat_found

      else : 
        self.subtext_nodata    = figure.add_subplot(1,1,1)
        #print "found no spectrum data"
      #print "nrows=",nrows,row

          
    # This is the function that makes the plots.
    # Add new plots or delete them using the given scheme.
    # The data has been already stored in self.plot_x, self.plot_xdif, etc ...
    # It is mandatory to add a tooltip variable to each subplot.
    # One might be tempted to merge addSubplots() and plotProductsGraphics().
    # There is a reason not to do it: addSubplots() is called only once at
    # startup, while plotProductsGraphics() is called always there is a resize.
    def plotProductsGraphics(self):
      #print "arc check:",  self.resid_tab_found, self.frameON_found
      #print "mfat check:",  self.edges_tab_found, self.mflat_found
      if (self.resid_tab_found == True and self.frameON_found == True) or (self.edges_tab_found == True and self.mflat_found == True): 
        #and self.lineguess_found == True 
        title_pref = 'Linear-extracted and Merged Spectrum.'	      

        tooltip_spectrum ="""\
        Plot of the linear-extracted and merged spectrum of the object 
        (blue line) as total flux (ADU) versus wavelength (nm). 
        The  +-1 sigma uncertainties are plotted as upper and lower
        red curves. Note that this spectrum is not flux calibrated.
        """


	if self.arm_uvb_found == True:
      	  title_spectrum   = title_pref

	if self.arm_vis_found == True:
      	  title_spectrum   = title_pref

	if self.arm_nir_found == True:
      	  title_spectrum   = title_pref
	   
	   
        #Display ON frame

        #Overplot initial list of lines
        #Overplot gauss list of lines
        #Overplot final list of lines

        #Scatter plot ResidX(lambda)
        #Scatter plot ResidY(lambda)

        #Scatter plot X(lambda)
        #Scatter plot Y(lambda)


        #Image
        spec_frameON   = ''
        if self.rec_id[4:12] == "predict" or self.rec_id[4:9] == "2dmap" :

           spec_frameON   = 'Arc lamp pinhole '
           tooltip_frameON ="""\
           Bias (and dark) corrected arc lamp pinhole
           image. Blue points are the predicted positions of the arc lines
           in the reference arc line list using the initial physical
           model. Yellow points are the positions of those predicted
           lines that are successfully found and fitted by a two-dimensional
           Gaussian. Green points are the line positions which remain after
           thresholding the Gaussian fits for signal-to-noise and
           refining the physical model fit.  
           A red hue corresponds to flagged pixels, which
           may be good or bad.
          """

        elif self.rec_id[4:12] == "orderpos" :
           spec_frameON   = 'Flat lamp pinhole '
           tooltip_frameON ="""\
           Bias (and dark) corrected flat lamp pinhole
           image. Green points trace each detected order. 
           A red hue corresponds to flagged pixels, which
           may be good or bad.
           """
        
        title_frameON   = spec_frameON+'Image'

        title_mflat   = 'Master flat frame'
        tooltip_mflat ="""\
                Image of the bias (dark) corrected master flat
                frame. Over are displayed order central traces (blue), and
                order detected  edges (left-yellow, right-green). 
                A red hue corresponds to flagged pixels, which
                may be good or bad.
                """


        title_plot_residX   = 'X Residual vs wavelength ' 
        tooltip_plot_residX ="""\
                             X Residual vs wavelength."""

        title_plot_residY   = 'Y Residual vs wavelength ' 
        tooltip_plot_residY ="""\
                             Y Residual vs wavelength."""

        title_plot_X   = 'X vs wavelength ' 
        tooltip_plot_X ="""\
                        X vs wavelength."""

        title_plot_Y   = 'Y vs wavelength ' 
        tooltip_plot_Y ="""\
                        Y vs wavelength."""


        #2D frame PRE format (predict/orderpos/2dmap/wavecal/flexcomp)
        #print "arc check:",  self.resid_tab_found, self.frameON_found
        if self.resid_tab_found == True and self.frameON_found == True:
           if self.rec_id[4:12] == "predict" or self.rec_id[4:9] == "2dmap" :
              self.plotImageScatter3Col(self.frameON,self.bpm,self.resid_tab,1, self.binx,self.biny,'X [pix]', 'Y [pix]', 'Xthpre', 'Ythpre', 'XGauss', 'YGauss', 'Xthanneal', 'Ythanneal', self.subplot_frameON,title_frameON,tooltip_frameON)

           if self.rec_id[4:12] == "orderpos" :

              self.plotImageScatter1Col(self.frameON,self.bpm,self.resid_tab,1, self.binx,self.biny,'X [pix]', 'Y [pix]', 'X', 'Y', self.subplot_frameON,title_frameON,tooltip_frameON)


        #2D master flat (mflat (slit/ifu))
        #print "mfat check:",  self.edges_tab_found, self.mflat_found
        if self.edges_tab_found == True and self.mflat_found  == True :
             self.plotImageScatter3Col(self.mflat,self.bpm,self.edges_tab,
        2, self.binx, self.biny,'X [pix]', 'Y [pix]', 'CENTER_X', 'CENTER_Y', 'EDG_LO_X', 'CENTER_Y','EDG_UP_X', 'CENTER_Y', self.subplot_mflat,title_mflat,tooltip_mflat)


        #print 'edges_found',self.edges_tab_found,'resid_tab_found',self.resid_tab_found,'rec_id',self.rec_id,'arm_uvb_found',self.arm_uvb_found
        #Scatter plot: set titles and labels
        if self.resid_tab_found == True  and \
          (self.rec_id[4:12] == "predict" or \
           self.rec_id[4:9] == "2dmap") :

           title_resid_x_tab   = 'Line X residuals.' 
           tooltip_resid_x_tab ="""\
                Plot of the line position residuals in X versus wavelength."""

           title_resid_y_tab   = 'Line Y residuals.' 
           tooltip_resid_y_tab ="""\
                 Plot of the line position residuals in Y versus wavelength."""

        elif self.resid_tab_found == True  and \
             self.rec_id[4:13] == "orderpos" :

           title_resid_x_tab   = 'Trace X residuals vs X' 
           tooltip_resid_x_tab ="""\
                                Plot of trace position X residuals versus X."""
           title_resid_y_tab   = 'Trace X residuals vs Y' 
           tooltip_resid_y_tab ="""\
                                Plot of trace position X residuals versus Y."""

        #Scatter plot: make the plot
        if self.resid_tab_found == True  and \
          (self.rec_id[4:12] == "predict" or \
           self.rec_id[4:9] == "2dmap" or \
           self.rec_id[4:13] == "orderpos") :
            if self.rec_id[4:12] == "predict" or self.rec_id[4:9] == "2dmap" :
               #print "resid tab"
               self.plotTableScatter(self.resid_tab,
        'Wavelength','ResidXModel', 'Wavelength [nm]','Line position residuals in X [pix]',self.subplot_resid_x_tab,title_resid_x_tab, tooltip_resid_x_tab)
               self.plotTableScatter(self.resid_tab, 'Wavelength','ResidYModel', 'Wavelength [nm]','Line position residuals in Y [pix]',self.subplot_resid_y_tab,title_resid_y_tab, tooltip_resid_y_tab)
            elif self.rec_id[4:13] == "orderpos" :
               self.plotTableScatter(self.resid_tab, 'X','RESX', 'X Position [pix]','X Residuals [pix]',self.subplot_resid_x_tab,title_resid_x_tab, tooltip_resid_x_tab)
               self.plotTableScatter(self.resid_tab, 'Y','RESX', 'Y Position [pix]','X Residuals [pix]',self.subplot_resid_y_tab,title_resid_y_tab, tooltip_resid_y_tab)
 
      else :
        #Data not found info
        self.subtext_nodata.set_axis_off()
        self.text_nodata = """\
                           ON-frame, Residual table 
                           (PRO.CATG=PREF_ON_ARM,PREF_RESID_ARM, 
                           where PREF=FMTCHK, ORDEF, FLAT, or WAVE, 
                           ARM=UVB,VIS or NIR) not found in the products. 
                           This may be due to a recipe failure. 
                           Check your input parameter values, 
                           correct possibly typos and
                           press 'Re-run recipe' button."""
        self.subtext_nodata.text(0.1, 0.6, self.text_nodata, color='#11557c', 
                                 fontsize=18, ha='left', va='center', alpha=1.0)
        self.subtext_nodata.tooltip="""\
                                    Merged spectrum not found in the products"""
        #print "found no spectrum data"
 
  
    # This function specifies which are the parameters that should be presented
    # in the window to be edited.
    # Note that the parameter has to be also in the in_sop port (otherwise it 
    # won't appear in the window) 
    # The descriptions are used to show a tooltip. They should match one to one
    # with the parameter list 
    # Note also that parameters have to be prefixed by the 'recipe name:'

    def setInteractiveParameters(self):
      paramList = list()
      rec_id=self.rec_id
      self.par = xsh_parameters_common.Parameters()
      self.par.setGeneralCalibParameters(paramList,rec_id)
  
      if rec_id[4:11] == "predict" :
         self.par.setDetectArclinesParameters(paramList,rec_id)
         self.par.setModelParameters(paramList,rec_id)

      elif rec_id[4:12] == "orderpos" :
         self.par.setDetectContinuumParameters(paramList,rec_id)

      elif rec_id[4:9] == "mflat" :
         self.par.setStackParameters(paramList,rec_id)
         self.par.setDetectOrderParameters(paramList,rec_id)
         self.par.setBackgroundParameters(paramList,rec_id)

      elif rec_id[4:9] == "2dmap" :
         self.par.setDetectArclinesParameters(paramList,rec_id)
         self.par.setModelParameters(paramList,rec_id)

      elif rec_id[4:11] == "wavecal" :
         print "found wavecal params"

      elif rec_id[4:12] == "flexcomp" :
         print "found flexcomp params"
      else :
         print "recipe" ,rec_id,  "not supported"

      return paramList

    def setWindowHelp(self):
      help_text = """
This is an interactive window which help asses the quality of the execution of a recipe.
"""
      return help_text

    def setWindowTitle(self):
      title = 'X-shooter Interactive ' + self.rec_subtitle + self.seq_arm + ' Arm. ' 
      return title

except ImportError:
  import_sucess = 'false'
  print "Error importing modules pyfits, wx, matplotlib, numpy"
  raise




# import the needed modules
try:
#  import sys
  import numpy
  try:
    from astropy.io import fits as pyfits
  except:
    import pyfits

  import wx
  import matplotlib
  import pipeline_product
  import xsh_plot_widgets

  from reflex import parseSofJson,RecipeParameter
  from reflex_interactive_app import PipelineInteractiveApp
  from pipeline_product import PipelineProduct
  from pipeline_display import SpectrumDisplay, ImageDisplay, ScatterDisplay

  import xsh_parameters_common

#  matplotlib.use('WXAgg')
  import_sucess = 'true'
  import warnings
  warnings.simplefilter('ignore',UserWarning)

#NOTE for developers: 
# -If you want to modify the current script to cope
#  with different parameters, this is the function to modify:
#  setInteractiveParameters()
# -If you want to modify the current script to read different data from
#  the input FITS, these are the functions (class DataPlotterManager) to modify:
#  readFitsData()    to indicate what are the FITS input to plot
#  plotSpectrum()    to control 1D image product layout
#  plotImage()       to control 2D image product layout
#  plotTable()       to control 1D table product layout
#  oplotTable()      to control 1D table product overplots
#  getSN()           to access 1D spectrum S/N FITS header info
#  setInsMode()      to access ins mode FITS header info
#  setObjType()      to access obj type FITS header info
#  setRecId()        to set rec id (used in parameter prefix)
#  getArmId()        to get arm id FITS header info
#
# -If you want to modify the current script to modify the plots (using the same
#  data),  this is the function to modify:
#  plotProductsGraphics()          (from class DataPlotterManager)
# -If you want to modify the text that appears in the "Help" button,
#  this is the function to modify:
#  setWindowHelp()
# -If you want to modify the title of the window, modify this function:
#  setWindowTitle()


  #This class deals with the specific details of data reading and final plotting.
  class DataPlotterManager:

    def paragraph(self,text, width=None):
       """ wrap text string into paragraph
           text:  text to format, removes leading space and newlines
           width: if not None, wraps text, not recommended for tooltips as
                  they are wrapped by wxWidgets by default
       """
       import textwrap
       #print 'width=',width,'Text=',text
       if width is None:
           return textwrap.dedent(text).replace('\n', ' ').strip()
       else:
           return textwrap.fill(textwrap.dedent(text), width=width) 


    #1D spectrum plots
    def plotSpectrum(self,data,errs,qual,bpcode,subplot,title,tooltip):
        data.readSpectrum(0)
        errs.readSpectrum(1)
        nwave = len(data.flux)
        minlimwave = int(nwave*0.25)
        maxlimwave = int(nwave*0.75)

        spectrum = SpectrumDisplay()
        spectrum.setLabels('Wavelength [nm]','Total Flux ['+data.bunit+']')

        spectrum.display(subplot,title,self.paragraph(tooltip),data.wave,data.flux,errs.flux, autolimits = True) 
        #matplotlib.pyplot.tight_layout()
        if self.response_found is True and self.rec_id[16:19] != "nod":
           #in case response is available we need to reduce fonts
           subplot.set_title(title, fontsize=7)
        try:
            subplot.figure.tight_layout()
        except AttributeError:
            pass

        #Flag BP as errors
        qual.readSpectrum(2)
        bpm_flag = numpy.bitwise_and(qual.flux,int(bpcode) ) == 0 
        mask_f_data=numpy.ma.MaskedArray(data.flux, mask=bpm_flag).compressed()
        mask_f_wave=numpy.ma.MaskedArray(data.wave,mask=bpm_flag).compressed()
        size_f_err=numpy.ma.MaskedArray(errs.flux,mask=bpm_flag).compressed()

        bpm_all = (qual.flux == 0 ) 
        mask_a_data=numpy.ma.MaskedArray(data.flux, mask=bpm_all).compressed()
        mask_a_wave=numpy.ma.MaskedArray(data.wave,mask=bpm_all).compressed()
        size_a_err=numpy.ma.MaskedArray(errs.flux,mask=bpm_all).compressed()
        #print "size all", mask_a_data.size
        if mask_a_data.size > 0 :
           subplot.errorbar(mask_a_wave, mask_a_data, xerr=None, yerr=size_a_err, color='k', linestyle='', capsize=0)
        #print "size masked", mask_f_data.size
        #print "size spectrum", data.flux.size
        if mask_f_data.size > 0 :
           subplot.errorbar(mask_f_wave, mask_f_data, xerr=None, yerr=size_f_err, color='r', linestyle='', capsize=0)
        subplot.set_xlim(spectrum.wave_lim)
        subplot.set_ylim(spectrum.flux_lim)
       

    #2D image display
    def plotImage(self,obj,bpm,subplot,title,tooltip):

          obj.readImage(fits_extension=0)
          obj.read2DLinearWCS(fits_extension=0)
          img_obj = ImageDisplay()

          bpm.readImage(fits_extension=2)
          img_bpm = ImageDisplay()

	  img_obj.setLabels('Wavelength [nm]','Position Along Slit \n [arcsec]')
          y_size, x_size = obj.image.shape
          #print 'x_size=',x_size , 'y_size=',y_size
          #print 'crval1=',obj.crval1,'crpix1=',obj.crpix1,'cdelt1=',obj.cdelt1 
          
          img_obj.setXLinearWCSAxis(obj.crval1,obj.cdelt1,obj.crpix1)
          img_obj.setYLinearWCSAxis(obj.crval2,obj.cdelt2,obj.crpix2)

	  #img_obj.setLimits((0,x_size),(0,y_size))
          img_obj.display(subplot, title, self.paragraph(tooltip),obj.image,bpm.image)
          if self.response_found is True and self.rec_id[16:19] != "nod":
             #in case response is available we need to reduce fonts
             subplot.set_title(title, fontsize=7)



    #3D cube slices display
    def plotCubeSlice(self,obj,bpm,slice,subplot,title,tooltip):
          obj.readImage(fits_extension=0)
          obj.read2DLinearWCS(fits_extension=0)
          img_obj = ImageDisplay()
          image = obj.image[slice,:,:]

          bpm.readImage(fits_extension=2)
          img_bpm = ImageDisplay()
          error = obj.image[slice,:,:]

	  #img_obj.setLabels('IFU slice [arcsec]','Position Along Slit \n [arcsec]')
	  img_obj.setLabels('IFU slice [arcsec]','Position Along Slit \n [pixel]')
          y_size, x_size = image.shape
          print 'x_size=',x_size , 'y_size=',y_size
          #print 'crval1=',obj.crval1,'crpix1=',obj.crpix1,'cdelt1=',obj.cdelt1 
          #print 'crval2=',obj.crval2,'crpix2=',obj.crpix2,'cdelt2=',obj.cdelt2 
          
          #img_obj.setXLinearWCSAxis(self.crval1,self.cdelt1,self.crpix1)
          #img_obj.setYLinearWCSAxis(self.crval1,self.cdelt1,self.crpix1)
          img_obj.setXLinearWCSAxis(-2,2,1)
          #img_obj.setYLinearWCSAxis(self.crval1,self.cdelt1,self.crpix1)

	  #img_obj.setLimits((0,x_size),(0,y_size))
          img_obj.display(subplot, title, self.paragraph(tooltip),image)
          if self.response_found is True and self.rec_id[16:19] != "nod":
             #in case response is available we need to reduce fonts
             subplot.set_title(title, fontsize=7)


    #1D spectrum table plots
    def plotTable(self,tab,Xcol,Ycol,Xlab,Ylab,subplot,title,tooltip,
    wave = None):

        tab.readTableXYColumns(fits_extension=1,xcolname=Xcol, ycolname=Ycol)
        spectrum = SpectrumDisplay()
        plotmax = numpy.nanmax(tab.y_column)
        plotmin = numpy.nanmin(tab.y_column)
    

        flux_plotmax = plotmax + 0.2 * plotmax
        flux_plotmin = plotmin - 0.2 * plotmax
      
        spectrum.setLabels(Xlab,Ylab)
        spectrum.flux_lim = flux_plotmin, flux_plotmax
        if wave is not None:
           spectrum.setWaveLimits((wave[0], wave[len(wave)-1]))

        spectrum.display(subplot,title,self.paragraph(tooltip),tab.x_column,tab.y_column) 
        if self.response_found is True and self.rec_id[16:19] != "nod":
           #in case response is available we need to reduce fonts
           subplot.set_title(title, fontsize=7)

    #1D spectrum table over-plots
    def oplotTable(self,tab,Xcol,Ycol,Xlab,Ylab,subplot,color):

        tab.readTableXYColumns(fits_extension=1,xcolname=Xcol, ycolname=Ycol)
        wave_lim = subplot.get_xlim()
        flux_lim = subplot.get_ylim()
        spectrum = SpectrumDisplay()
        spectrum.overplot(subplot,tab.x_column,tab.y_column,color)
        subplot.set_xlim(wave_lim)
        subplot.set_ylim(flux_lim)

    def scatterPlotScatter(self, subplot, x, y,color=None):
        if color == None :
           subplot.scatter(x, y, s=0.4, color='blue')
        else:
           subplot.scatter(x, y, s=0.4, color=color)

        subplot.ticklabel_format(style='sci',scilimits=(0,3))

    def scatterPlotLabels(self, subplot, x_label, y_label):
        subplot.set_xlabel(x_label, fontsize=12)
        subplot.set_ylabel(y_label, fontsize=12)

    def finalSetup(self, subplot, title, tooltip):
        subplot.grid(True)
        subplot.set_title(title, fontsize=12)
        subplot.tooltip = self.paragraph(tooltip)

    def scatterPlot(self, subplot, x, y, x_label, y_label, title, tooltip):

        self.scatterPlotScatter(subplot, x, y)
        self.scatterPlotLabels(subplot, x_label, y_label)
        self.scatterPlotLimits(subplot, x, y)
        self.finalSetup(subplot, title, self.paragraph(tooltip))

    def scatterPlotLimits(self, subplot, x, y):
        subplot.set_xlim(numpy.nanmin(x)-1, numpy.nanmax(x) + 1.)
        subplot.set_ylim(1.05 * numpy.nanmin(y), 1.05 * numpy.nanmax(y))


    #1D table scatter plots
    def plotTableScatter(self,tab,Xcol,Ycol,Xlab,Ylab,subplot,title,tooltip,ext_no=None):
        extno=1
        if ext_no != None :
           extno=ext_no

        tab.readTableXYColumns(fits_extension=extno,xcolname=Xcol,ycolname=Ycol)
        self.scatterPlotScatter(subplot, tab.x_column, tab.y_column)
        self.scatterPlotLabels(subplot, Xlab, Ylab)
        self.scatterPlotLimits(subplot, tab.x_column, tab.y_column)
        self.finalSetup(subplot, title, self.paragraph(tooltip))
        self.scatterPlot(subplot, tab.x_column, tab.y_column, Xlab, Ylab, title, tooltip)


    def oplotTableScatter(self,tab,Xcol,Ycol,subplot,ext_no=None,color=None):
        extno=1
        if ext_no != None :
           extno=ext_no

        tab.readTableXYColumns(fits_extension=extno,xcolname=Xcol,ycolname=Ycol)
        self.scatterPlotScatter(subplot, tab.x_column, tab.y_column,color=color)

        #subplot.scatter(tab.x_column, tab.y_column,color)

    def getDecodeBP(self, fitsfile):
        hdulist = pyfits.open(fitsfile.name)
        self.decode_bp = hdulist[0].header['ESO PRO REC1 PARAM4 VALUE']

    #Get NAXIS info (Xshooter and arm specific)
    def getNAXIS3(self, fitsfile):
      hdulist = pyfits.open(fitsfile.name)
      self.NAXIS3 = hdulist[0].header['NAXIS3']

    def getWCS(self, fitsfile):
      hdulist = pyfits.open(fitsfile.name)
      self.NAXIS1 = hdulist[0].header['NAXIS1']
      self.NAXIS2 = hdulist[0].header['NAXIS2']
      self.NAXIS3 = hdulist[0].header['NAXIS3']

      self.crpix1 = hdulist[0].header['CRPIX1']
      self.crval1 = hdulist[0].header['CRVAL1']
      self.cdelt1 = hdulist[0].header['CDELT1']

      self.crpix2 = hdulist[0].header['CRPIX2']
      self.crval2 = hdulist[0].header['CRVAL2']
      self.cdelt2 = hdulist[0].header['CDELT2']

      self.crpix3 = hdulist[0].header['CRPIX3']
      self.crval3 = hdulist[0].header['CRVAL3']
      self.cdelt3 = hdulist[0].header['CDELT3']
      self.wmax= self.crval3+(self.NAXIS3-1)*self.cdelt3
    #Get S/N info (Xshooter and arm specific)
    def getSN(self, fitsfile):
      hdulist = pyfits.open(fitsfile.name)
      self.SN1 = hdulist[0].header['ESO QC FLUX1 SN']
      if self.arm_uvb_found  == True :
        self.SN2 = hdulist[0].header['ESO QC FLUX2 SN']
        self.RN1 = "450-470 nm"
        self.RN2 = "510-530 nm"
      if self.arm_vis_found  == True :
        self.SN2 = hdulist[0].header['ESO QC FLUX2 SN']
        self.SN3 = hdulist[0].header['ESO QC FLUX3 SN']
        self.RN1 = "672-680 nm"
        self.RN2 = "745-756 nm"
        self.RN3 = "992-999 nm"
      if self.arm_nir_found  == True :
        self.RN1 = "1514-1548 nm"
        self.OPTI5_NAME = hdulist[0].header['ESO INS OPTI5 NAME']
        if "JH" in self.OPTI5_NAME :
   	  self.arm_nir_jh_found = True
          print "Found NIR JH filter"
        else :
          self.SN2 = hdulist[0].header['ESO QC FLUX2 SN']
          self.RN2 = "2214-2243 nm"
     
    # This function will read all the columns, images and whatever is needed
    # from the products. The variables , self.plot_x, self.plot_y, etc...
    # are used later in function plotProductsGraphics().
    # Add/delete these variables as you need (only that plotProductsGraphics()
    # has to use the same names).
    # You can also create some additional variables (like statistics) after
    # reading the files.
    # If you use a control variable (self.xxx_found), you can modify 
    # later on the layout of the plotting window based on the presence of 
    # given input files. 
    # sof contains all the set of frames
    def readFitsData(self, fitsFiles):
      #Control variable to check if the interesting files where at the input
      prefix = "SCI_IFU_"
      self.spectrum3D_found   = False
      self.traces_ifu_slices_found   = False
      self.spectrum1D_found   = False
      self.flux_spectrum1D_found   = False
      self.lineguess_found = False
      self.spectrum2D_found      = False
      self.response_found      = False
      self.mresponse_found      = False
      self.sky_spectrum2D_found      = False
      self.sky2D_found   = False
      self.obj_sky2D_found   = False
      self.bpm_found   = False
      self.arm_uvb_found      = False
      self.arm_vis_found      = False
      self.arm_nir_found      = False
      self.arm_nir_jh_found   = False
      self.nima = 0
      self.NAXIS3=100

      #Read all the products
      frames = dict()
      for frame in fitsFiles:
        if frame == '' :
          continue
        #hdulist = pyfits.open(frame)
        #hdulist = frame.hdulist()
        header = pyfits.open(frame.name)
        #Make sure to have only products from the same recipe
        #that used this (common) script 
        if 'ESO PRO REC1 ID' in header[0].header :
           rec_id = header[0].header['ESO PRO REC1 ID']
           if rec_id == self.rec_id :
              #print rec_id, self.rec_id
              category = frame.category
              frames[category] = frame
              print frame
      #Note that you have to take care that no other input to this py script
      #the same prefix as SCI_IFU, TELL_IFU, FLUX_IFU
 
      prefixes = [k.split('_')[0]+"_"+k.split('_')[1] for k in frames.keys()]
      print  "prefixes=",prefixes
      if "SCI_IFU" in prefixes :
        prefix = "SCI_IFU_"
      elif "TELL_IFU" in prefixes:
        prefix = "TELL_IFU_"
      elif "FLUX_IFU" in prefixes:
        prefix = "FLUX_IFU_"
      elif "CAL_IFU" in prefixes:
        prefix = "CAL_IFU_"
      elif "SKY_IFU" in prefixes:
        prefix = "SKY_IFU_"

      arm=self.seq_arm

      if arm == "UVB":
 	 self.arm_uvb_found = True
      if arm == "VIS":
 	 self.arm_vis_found = True
      if arm == "NIR":
 	 self.arm_nir_found = True
      print "prefix=",prefix
# For any arm search a list of input frames
      key = prefix+"MERGE3D_DATA_OBJ_"+arm
      if key in frames :
        self.spectrum3D_found = True
        hdulist = frames[prefix+"MERGE3D_DATA_OBJ_"+arm]
        self.hdu3D = hdulist
        self.spectrum3D = PipelineProduct(hdulist)
        self.error3D = PipelineProduct(hdulist)
        self.qual3D = PipelineProduct(hdulist)
        self.getSN(hdulist)
        self.getNAXIS3(hdulist)
        self.getWCS(hdulist)
        #print "SN=",self.SN1
        self.getDecodeBP(hdulist)
        #print "Decode BP=",self.decode_bp
        self.nima += 1
        #print "found MERGED3D spectrum"

      key = prefix+"MERGE3D_TRACE_OBJ_"+arm
      if key in frames :
        self.traces_ifu_slices_found = True
        hdulist = frames[prefix+"MERGE3D_TRACE_OBJ_"+arm]
        self.traces_ifu_slices = PipelineProduct(hdulist)
        self.nima += 1
        #print "found MERGED3D Trace"

      key = prefix+"MERGE1D_"+arm
      if key in frames :
        self.spectrum1D_found = True
        hdulist = frames[prefix+"MERGE1D_"+arm]
        #hdulist = hdulist[prefix+"MERGE1D_"+arm]
        self.spectrum1D = PipelineProduct(hdulist)
        self.error1D = PipelineProduct(hdulist)
        self.qual1D = PipelineProduct(hdulist)
        self.getSN(hdulist)
        #print "SN=",self.SN1
        self.getDecodeBP(hdulist)
        #print "Decode BP=",self.decode_bp
        self.nima += 1
        #print "found MERGED1D spectrum"
      
      key = prefix+"FLUX_MERGE1D_"+arm
      if key in frames :
        self.flux_spectrum1D_found = True
        hdulist = frames[prefix+"FLUX_MERGE1D_"+arm]
        self.flux_spectrum1D = PipelineProduct(hdulist)
        self.flux_error1D = PipelineProduct(hdulist)
        self.flux_qual1D = PipelineProduct(hdulist)
        self.nima += 1
        #print "found FLUX_MERGED1D spectrum"
	
      key = prefix+"MERGE2D_"+arm
      if key in frames :
        self.spectrum2D_found = True
        hdulist = frames[prefix+"MERGE2D_"+arm]
        self.bpm = PipelineProduct(hdulist)
        self.spectrum2D = PipelineProduct(hdulist)
        self.nima += 1
        #print "found MERGED2D spectrum"
        header = pyfits.open(hdulist.name)
        self.key_object = header[0].header['OBJECT']
        #print prefix
        #print object
        if 'CAL_IFU_' in prefix and 'SKY' in self.key_object and 'stare' in self.rec_id :
            self.obj_type = "Sky"

      key = "SKY_IFU_MERGE2D_"+arm
      if key in frames and self.rec_id[16:19] != "nod":
        self.sky_spectrum2D_found = True
        hdulist = frames["SKY_IFU_MERGE2D_"+arm]
        self.sky_spectrum2D = PipelineProduct(hdulist)
        self.nima += 1
        #print "found SKY_IFU_MERGED2D spectrum"

      key = prefix+"SKY_"+arm
      if key in frames :
        self.sky2D_found = True
        hdulist = frames[prefix+"SKY_"+arm]
        self.sky2D = PipelineProduct(hdulist)
        #print "found SKY_2D spectrum"

      key = prefix+"SUB_SKY_"+arm
      if key in frames :
        self.obj_sky2D_found = True
        hdulist = frames[prefix+"SUB_SKY_"+arm]
        self.obj_sky2D = PipelineProduct(hdulist)
        #print "found SUB_SKY spectrum"

      if self.rec_id[0:10] == "xsh_respon":

         key = "RESPONSE_MERGE1D_IFU_"+arm
         if key in frames : 
            self.response_found = True
            hdulist = frames["RESPONSE_MERGE1D_IFU_"+arm]
            self.response = PipelineProduct(hdulist)
            self.nima += 1
            #print "found RESPONSE_MERGE1D_IFU_ spectrum"

         key = "MRESPONSE_MERGE1D_IFU_"+arm
         if key in frames : 
            self.mresponse_found = True
            hdulist = frames["MRESPONSE_MERGE1D_IFU_"+arm]
            self.mresponse = PipelineProduct(hdulist)
            #print "found MRESPONSE_MERGE1D_IFU_ spectrum"

    #get ins mode (Slit/IFU and Stare/Offset/Nodding)
    def setInsMode(self, rec_id):
      #Recipe ID variable to properly define params
      self.rec_id = rec_id

      if self.rec_id[16:21] == "stare" :
         self.ins_mode = "Slit-Stare"
      elif self.rec_id[16:22] == "offset" :
         self.ins_mode = "Slit-Offset"

      elif self.rec_id[16:19] == "nod" :
         self.ins_mode = "Slit-Nodding"
      elif self.rec_id == "xsh_scired_ifu_stare" :
         self.ins_mode = "ifu-stare"
      elif self.rec_id == "xsh_scired_ifu_offset" :
         self.ins_mode = "ifu-offset"

    #Set Object type (Object/Std star) to display proper info
    def setObjType(self, rec_id):
      #Recipe ID variable to properly define params
      self.rec_id = rec_id

      if self.rec_id[0:10] == "xsh_scired":
         self.obj_type = "Object"
      
      if self.rec_id[0:10] == "xsh_respon":
         self.obj_type = "Std Star"



    #Set rec id (to have proper recipe parameters prefix)
    def setRecId(self, rec_id):
      #Recipe ID variable to properly define params
      self.rec_id = rec_id
      self.setInsMode(rec_id)
      self.setObjType(rec_id)

    #Get arm setting
    def getArmId(self, sof):
      #Recipe ID variable to properly define params
      self.rec_id = "xsh_respon_slit_stare"
      self.seq_arm = "UVB"
      nf = 0

      frames = dict()
      files = sof.files
      for f in files:
      #for frame in fitsFiles:
        file_name=f.name
        if file_name == '' :
          continue
        else :
           nf += 1
           hdulist = pyfits.open(file_name)
           rec_id_list = hdulist[0].header['ESO PRO REC1 ID']
           arm_list = hdulist[0].header['ESO SEQ ARM']
      if nf != 0 :
         self.rec_id = rec_id_list[0:]
         self.seq_arm = arm_list[0:]

      if self.seq_arm == "UVB":
         self.arm_uvb_found = True

      if self.seq_arm == "VIS":
         self.arm_uvb_found = True

      if self.seq_arm == "NIR":
         self.arm_uvb_found = True

      if self.rec_id[16:21] == "stare" :
         self.ins_mode = "Slit-Stare"
      elif self.rec_id[16:22] == "offset" :
         self.ins_mode = "Slit-Offset"

      elif self.rec_id[16:19] == "nod" :
         self.ins_mode = "Slit-Nodding"
      elif self.rec_id == "xsh_scired_ifu_stare" :
         self.ins_mode = "ifu-stare"
      elif self.rec_id == "xsh_scired_ifu_offset" :
         self.ins_mode = "ifu-offset"

      if self.rec_id[0:10] == "xsh_scired":
         self.obj_type = "Object"
      
      if self.rec_id[0:10] == "xsh_respon":
         self.obj_type = "Std Star"


    def plotWidgets(self) :
        widgets = list()
        min=0
        max=100
        title_slider="Select plane id by clicking with mouse left button"
        if self.spectrum3D_found == True :
           max=self.NAXIS3 
           val=numpy.floor(self.NAXIS3/2+0.5)
           # Cube plane selector 
           self.slider =xsh_plot_widgets.InteractiveSlider(self.cube_plane_selector,self.setPlaneSelectCallBack,min,max,val,0, title=title_slider)
           widgets.append(self.slider)

        return widgets

    def setPlaneSelectCallBack(self, plane_id) :
        plane_id = int(plane_id)
        self.subplot_cube.cla()
        self.setDisplayCubeSlice(plane_id)

    def setDisplayCubeSlice(self, plane_id) :
        self.wcen= self.crval3+(plane_id-1)*self.cdelt3
        self.title_cube_obj   = 'IFU slices plane id: %#6.5g wavelength: %#6.5g [nm]'% (plane_id,self.wcen) 

        self.plotCubeSlice(self.spectrum3D,self.spectrum3D,plane_id,self.subplot_cube,self.title_cube_obj,self.tooltip_cube_obj)



    # This function creates all the subplots. It is responsible for the plotting 
    # layouts. 
    # There can different layouts, depending on the availability of data
    # Note that subplot(I,J,K) means the Kth plot in a IxJ grid 
    # Note also that the last one is actually a box with text, no graphs.
    def addSubplots(self, figure):
      nrows=self.nima
      row=1;
      #print "self.spectrum1D_found",self.spectrum1D_found
      #print "self.spectrum2D_found",self.spectrum2D_found
      #print "self.flux_spectrum1D_found",self.flux_spectrum1D_found
      #print "self.response_found",self.response_found
      #print "self.spectrum2D_found",self.spectrum2D_found
      #print "self.sky_spectrum2D_found",self.sky_spectrum2D_found

      if self.spectrum1D_found == True and self.spectrum2D_found == True:
        self.subplot_spectrum  = figure.add_subplot(nrows,1,row)
        row +=1

        if self.flux_spectrum1D_found == True :
           self.subplot_flux_spectrum = figure.add_subplot(nrows,1,row)
           row +=1

        if self.response_found == True:
           self.subplot_response    = figure.add_subplot(nrows,1,row)
           row +=1

        if self.spectrum2D_found == True :
           self.subplot_image_obj = figure.add_subplot(nrows,1,row)
           row +=1

        if self.rec_id[17:19] != "nod" :
          if self.sky_spectrum2D_found == True :
             self.subplot_image_sky = figure.add_subplot(nrows,1,row)
             row +=1
      elif self.spectrum3D_found == True and self.traces_ifu_slices_found == True:
        self.subplot_cube   = figure.add_subplot(nrows,1,row)
        row +=1
        self.subplot_traces_ifu_slices  = figure.add_subplot(nrows,1,row)
        row +=1
        self.cube_plane_selector = figure.add_axes([0.1, 0.505, 0.83, 0.01])
        self.cube_plane_pix_range = figure.add_axes([0.1, 0.495, 0.83, 0.01])
        self.cube_plane_wav_range = figure.add_axes([0.1, 0.465, 0.83, 0.01])

        # Set the colormap and norm to correspond to the data for which
        # the colorbar will be used.

        cmap = matplotlib.cm.afmhot
        norm1 = matplotlib.colors.Normalize(vmin=0, vmax=self.NAXIS3)
        norm2 = matplotlib.colors.Normalize(vmin=self.crval3, vmax=self.wmax)

        #add colorbar
        cb1 = matplotlib.colorbar.ColorbarBase(self.cube_plane_pix_range, cmap=cmap,
                                   norm=norm1,orientation='horizontal')
        cb2 = matplotlib.colorbar.ColorbarBase(self.cube_plane_wav_range, cmap=cmap,
                                   norm=norm2,orientation='horizontal')

      else : 
        self.subtext_nodata    = figure.add_subplot(1,1,1)
        print "found no spectrum data"


 
    # This is the function that makes the plots.
    # Add new plots or delete them using the given scheme.
    # The data has been already stored in self.plot_x, self.plot_xdif, etc ...
    # It is mandatory to add a tooltip variable to each subplot.
    # One might be tempted to merge addSubplots() and plotProductsGraphics().
    # There is a reason not to do it: addSubplots() is called only once at
    # startup, while plotProductsGraphics() is called always there is a resize.
    def plotProductsGraphics(self):
      if self.spectrum1D_found == True and self.spectrum2D_found == True: 
        #and self.lineguess_found == True 
        title_pref = 'Standard-extracted and Merged Spectrum.'	      
        title_flux_spectrum = 'Standard Extracted, Merged, and Flux Calibrated Spectrum.'
        tooltip_spectrum ="""\
                          Plot of the standard-extracted and merged spectrum of the object (blue line) as total flux (ADU) versus wavelength (nm). The +-1 sigma uncertainties are plotted as the light blue region encompassing the object spectrum (and bounded by the light grey spectra). Flagged pixels are marked with a dark-grey vertical bar stretching between the +- 1 sigma limits. Flagged bad pixels that are decoded as "bad pixels" are marked by a red vertical bar stretching between the +- 1 sigma limits. Note that this spectrum is not flux calibrated."""

        tooltip_flux_spectrum ="""\
                              Plot of the standard-extracted, merged,
                              and flux calibrated spectrum of the
                              object (blue line) as total flux (10^-16
                              erg/s/cm^2/A) versus wavelength
                              (nm). The +-1 sigma uncertainties are
                              plotted as the light blue region
                              encompassing the object spectrum (and
                              bounded by the light grey
                              spectra). Flagged pixels are marked with
                              a dark-grey vertical bar stretching
                              between the +- 1 sigma limits. Flagged
                              bad pixels that are decoded as "bad
                              pixels" are marked by a red vertical bar
                              stretching between the +- 1 sigma
                              limits. The reference standard star
                              spectrum from the catalog is overplotted for
                              comparison purposes as a green line."""

	if self.arm_uvb_found == True:
      	  title_spectrum   = title_pref+' \n SN: %#.2f (%#s); %#.2f (%#s) '% (self.SN1,self.RN1,self.SN2,self.RN2)

	if self.arm_vis_found == True:
      	  title_spectrum   = title_pref+' \n SN: %#.2f (%#s); %#.2f (%#s); %#.2f (%#s) '% (self.SN1,self.RN1,self.SN2,self.RN2,self.SN3,self.RN3)

	if self.arm_nir_found == True and self.arm_nir_found == True:
      	     title_spectrum   = title_pref+' \n SN: %#.2f (%#s); '% (self.SN1,self.RN1)
        elif self.arm_nir_found == True :
      	     title_spectrum   = title_pref+' \n SN: %#.2f (%#s); %#.2f (%#s); '% (self.SN1,self.RN1,self.SN2,self.RN2)
	   
	   
        #Spectrum plot

        self.plotSpectrum(self.spectrum1D,self.error1D,self.qual1D,self.decode_bp,self.subplot_spectrum,title_spectrum,tooltip_spectrum)

        #Flux calibrated Spectrum plot

        if self.flux_spectrum1D_found == True:

           self.plotSpectrum(self.flux_spectrum1D,self.flux_error1D,self.flux_qual1D,self.decode_bp,self.subplot_flux_spectrum,title_flux_spectrum,tooltip_flux_spectrum)
        if self.response_found == True :
           self.oplotTable(self.response,'LAMBDA','REF','Wavelength [nm]','Reference flux [erg/e-/cm^2]',self.subplot_flux_spectrum,'green')



        #Image
        title_image_obj   = '2D Merged ' + self.obj_type + ' Spectrum' 
        tooltip_image_obj ="""\
                           Image of the 2D merged spectrum of the
        object in ADU. A red hue corresponds to flagged pixels, which
        may be good or bad. Note that this spectrum is not flux calibrated."""


        title_image_sky   = '2D Merged Sky Spectrum ' 
        tooltip_image_sky ="""\
                           Image of the 2D merged spectrum of the sky
        in ADU. A red hue corresponds to flagged pixels, which
        may be good or bad. Note that this spectrum is not flux calibrated."""
        #Object merged 2D
        if self.spectrum2D_found  == True:
           self.plotImage(self.spectrum2D,self.bpm,self.subplot_image_obj,title_image_obj,tooltip_image_obj)


        #sky merged 2D
        if self.sky_spectrum2D_found  == True and self.rec_id != "xsh_respon_slit_nod" and self.rec_id != "xsh_scired_slit_nod" :

           self.plotImage(self.sky_spectrum2D,self.bpm,self.subplot_image_sky,title_image_sky,tooltip_image_sky)


        #Scatter plot
        if self.response_found == True :
           title_response   = 'Instrument response curve' 
           tooltip_response ="""\
                             Plot of the instrument response curve
        (erg/e-/cm^2) as determined from the standard star observation
        (green). The raw response is overplotted for comparison
        purposes (blue). If the master response curve is available, then it is overplotted for comparison purposes (yellow)."""
           if self.spectrum1D_found == True:
              self.plotTable(self.response,'LAMBDA','REF_DIV_OBS','Wavelength [nm]','Raw Response \n[erg/e-/cm^2]',self.subplot_response,title_response,tooltip_response,self.spectrum1D.wave)
              self.oplotTable(self.response,'LAMBDA','RESPONSE','Wavelength [nm]','Response [erg/e-/cm^2]',self.subplot_response,'green')

           else:
              #print "self.response_found"
              self.plotTable(self.response,'LAMBDA','RESPONSE','Wavelength [nm]','Response [erg/e-/cm^2]',self.subplot_response,title_response,tooltip_response)
           if self.mresponse_found == True :
              self.oplotTable(self.mresponse,'LAMBDA','RESPONSE','Wavelength [nm]','Response [erg/e-/cm^2]',self.subplot_response,'yellow')

      if self.spectrum3D_found == True and self.traces_ifu_slices_found :


        plane_id=numpy.floor(self.NAXIS3/2+0.5)
        #self.wcen= self.crval3+(plane_id-1)*self.cdelt3

        #self.title_cube_obj   = 'IFU slices plane %#6.0g %#6.3g'% (plane_id,self.wcen) 
        self.tooltip_cube_obj ="""\
                            The displayed image corresponds to the
        plane id indicated by the slider below. To view a different
        plane of the science reconstructed cube, the user should select
        the desired plane id with the mouse left button, using the
        slider shown below the image.  
                                      """
        title_traces_ifu_slices   = 'IFU slices traces' 
        tooltip_traces_ifu_slices ="""\
                          Traces of the IFU object central position
        observed by slicer number 1,2,3 are displayed respectively in
        colors blue, green, red. If the cube has been perfectly 
        reconstructed the three traces should overlap. Traces are well
        defined only for point-like and high SNR objects. Low SNR pr
        extended objects may present a not aggregated point distribution.
                                      """
        plane_id=numpy.floor(self.NAXIS3/2+0.5)
        self.setDisplayCubeSlice(plane_id)

        self.plotTableScatter(self.traces_ifu_slices,'WAVELENGTH','POS_1', 'Wavelength [nm]','Slit position [pix]',self.subplot_traces_ifu_slices,title_traces_ifu_slices,tooltip_traces_ifu_slices)
        self.oplotTableScatter(self.traces_ifu_slices,'WAVELENGTH','POS_2', self.subplot_traces_ifu_slices,color='green')
        self.oplotTableScatter(self.traces_ifu_slices,'WAVELENGTH','POS_3', self.subplot_traces_ifu_slices,color='red')


        #self.plotTable(self.traces_ifu_slices,'WAVELENGTH','POS_1','Wavelength [nm]','Slit position [pix]',self.subplot_traces_ifu_slices,title_traces_ifu_slices,tooltip_traces_ifu_slices)
        #self.oplotTable(self.traces_ifu_slices,'WAVELENGTH','POS_2','Wavelength [nm]','Slit position [pix]',self.subplot_traces_ifu_slices,'yellow')
        #self.oplotTable(self.traces_ifu_slices,'WAVELENGTH','POS_3','Wavelength [nm]','Slit position [pix]',self.subplot_traces_ifu_slices,'green')

      else :
        #Data not found info
        self.subtext_nodata.set_axis_off()
        self.text_nodata = """Merged spectrum
       (PRO.CATG=PREF_MERGE3D_DATA_OBJ_ARM,PREF_TRACE3D_DATA_OBJ_ARM, 
       where PREF=FLUX_IFU, TELL_IFU or SCI_IFU, 
       ARM=UVB,VIS or NIR) not found in the products. 
       This may be due to a recipe failure. 
      Check your input parameter values, 
      correct possibly typos and
      press 'Re-run recipe' button."""
        self.subtext_nodata.text(0.1, 0.6, self.text_nodata, color='#11557c', 
                                 fontsize=18, ha='left', va='center', alpha=1.0)
        self.subtext_nodata.tooltip="""\
                                    Merged spectrum not found in the products"""
        print "found no spectrum data"
 
  
    # This function specifies which are the parameters that should be presented
    # in the window to be edited.
    # Note that the parameter has to be also in the in_sop port (otherwise it 
    # won't appear in the window) 
    # The descriptions are used to show a tooltip. They should match one to one
    # with the parameter list 
    # Note also that parameters have to be prefixed by the 'recipe name:'
   	
    def setInteractiveParameters(self):
      paramList = list()

      rec_id=self.rec_id
      self.par = xsh_parameters_common.Parameters()

      self.par.setGeneralObjectParameters(paramList,rec_id)

      print rec_id
      if ("stare" in rec_id) or ("nod" in rec_id) :
         print "entered"
         self.par.setStackParameters(paramList,rec_id)
      if self.rec_id != "xsh_scired_ifu_stare" :
         self.par.setCrhSingleParameters(paramList,rec_id)

      if rec_id[16:21] == "stare" :
         print "background"
         self.par.setBackgroundParameters(paramList,rec_id)

      if self.rec_id != "xsh_scired_ifu_stare" :
         self.par.setLocalizeParameters(paramList,rec_id)
      self.par.setRectifyParameters(paramList,rec_id)

      return paramList

    def setWindowHelp(self):
      help_text = """
This is an interactive window which help asses the quality of the execution of a recipe.
"""
      return help_text

    def setWindowTitle(self):
      title = 'X-shooter Interactive ' + self.obj_type + ' Spectrum Extraction. ' + self.seq_arm + ' Arm. ' + self.ins_mode
      return title

except ImportError:
  import_sucess = 'false'
  print "Error importing modules pyfits, wx, matplotlib, numpy"
  raise




def log_invocation(fn='/tmp/reflex_invoc.log'):
    """ log invocation of actor for easy reexecution with:
        eval $(cat /tmp/reflex_invoc.log)
    """
    import os, sys
    #if 'REFLEX_DEBUG' not in os.environ:
    #    return
    try:
        with open(fn, 'w') as f:
            path = ""
            s = " ".join(['"%s"' % x for x in sys.argv])
            try:
                import reflex
                path = '"%s:"' % os.path.dirname(reflex.__file__)
            except ImportError:
                pass
            f.write('PYTHONPATH=%s$PYTHONPATH python %s\n' % (path, s))
    except:
        pass





# import the needed modules
try:
  import reflex
  import_sucess = 'true'

# Then here is python code moved in xsh_object_interactive_common_ifu.py

except ImportError:
  import_sucess = 'false'
  print "Error importing modules pyfits, wx, matplotlib, numpy"

#This is the 'main' function
if __name__ == '__main__':
  from xsh_log_invocation import *
  log_invocation()

  # import reflex modules
  from reflex import *
  from reflex_interactive_app import *
  from pipeline_display import *
  from pipeline_product import *
  from xsh_object_interactive_common_ifu import *

  # Create interactive application
  interactive_app = PipelineInteractiveApp(enable_init_sop=True)

  # PECULIAR XSH needs this in order to be able later to get from an input FITS
  # the ins-mode, arm (and recipe) IDs, used in titles and param setting
  # get inputs from the command line
  interactive_app.parse_args()
  inputs = interactive_app.inputs


  #Check if import failed or not
  if import_sucess == 'false' :
     interactive_app.setEnableGUI(false)
    
  #Open the interactive window if enabled
  if interactive_app.isGUIEnabled() :
    #Get the specific functions for this window
    dataPlotManager = DataPlotterManager()
    #print inputs.in_sof
    #dataPlotManager.checkSofIsNotEmpty(inputs.in_sof)
    #With the following call XSH get the: ins-mode, arm (and recipe) IDs
    dataPlotManager.getArmId(inputs.in_sof)
    #Set recipe ID in order to build proper param list, display layout
    dataPlotManager.setRecId("xsh_scired_ifu_offset")


    interactive_app.setPlotManager(dataPlotManager)
    interactive_app.showGUI()
  else :
    interactive_app.passProductsThrough()
   
  #Print outputs. This is parsed by the Reflex python actor to get the results
  #Do not remove
  interactive_app.print_outputs()
  sys.exit()
